#import "AppController.h"
#import <libpq-fe.h>
#import "writeReport.h"
#import "MyTableView.h"
#import <sys/time.h>

// When a notice is raised,  this function gets called
void handle_notice (void *arg, const char *message)
{
    AppController *me = (AppController *)arg;
    [me appendToLog:[NSString stringWithFormat:@"%s", message]
            color:[NSColor magentaColor]];
}

@implementation AppController

- (id)init
{
    [super init];
    
    // These are used for the history
    executedStatements = [[NSMutableArray alloc] init];
    currentIndex = 0;
	
	// allocate a writer object...
	myWriter = [[writeReport alloc] init];
	//[myWriter retain];  // Too used to autoreleased objects....
    
	NSMutableDictionary *defaultPrefs =[NSMutableDictionary dictionary];
	[defaultPrefs setObject:@"" forKey:@"BSQLuser"];
	[defaultPrefs setObject:@"" forKey:@"BSQLhost"];
	[defaultPrefs setObject:@"" forKey:@"BSQLdb"];
	NSData* inputFontAsData = [NSKeyedArchiver archivedDataWithRootObject:
	                             [NSFont fontWithName:@"Helvetica" size:12.0]];
	NSData* tableFontAsData =[NSKeyedArchiver archivedDataWithRootObject:
	                             [NSFont fontWithName:@"Lucida Grande" size:13.0]];

	[defaultPrefs setObject:inputFontAsData forKey:@"BSQLinputFont"];
	[defaultPrefs setObject:tableFontAsData forKey:@"BSQLtableFont"];

	prefs = [[NSUserDefaults standardUserDefaults] retain];
	[prefs registerDefaults:defaultPrefs];
	
    return self;
}

// locate where the user double clicked...
-(void) whereAmI
{
  // Hmmm, it works :)
  // NSLog(@"row: %d, col: %d",[tableView clickedRow],[tableView clickedColumn]);
}

// Get a list of all the types from the database
- (void)fetchTypes
{
    typeResult = PQexec(connection, 
        "select oid, typname from pg_type order by oid");
}

// Clear the previous result, hold onto the latest
- (void)setLastResult:(PGresult *)aResult
{
    // Release the last result
    if (lastResult) {
        PQclear(lastResult);
    }
    
    // Set a pointer to the new result
    lastResult = aResult;
}

// Logging methods

- (void)clearLog
{
    [outputTextView setString:@""];
    [outputTextView display];
}

- (void)appendToLog:(NSString *)s color:(NSColor *)c
{   
    NSFont *font = [outputTextView font];
    NSRange r;
    NSTextStorage *ts = [outputTextView textStorage];
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    [dict setObject:c forKey:NSForegroundColorAttributeName];
    [dict setObject:font forKey:NSFontAttributeName];
    NSAttributedString *toAdd = [[NSAttributedString alloc] initWithString:s attributes:dict];
    [dict release];
    [ts appendAttributedString:toAdd];
    [toAdd release];
    r.location = [ts length];
    r.length = 0;
    [outputTextView scrollRangeToVisible:r];
    [outputTextView display];
}

- (IBAction)saveEditWindow:(id)sender
{
  int result;
  NSString *tmpString;
  NSMutableString *preferredDir;
  
  preferredDir =[NSString stringWithFormat:@"%@%@",NSHomeDirectory(),@"/Library/BiggerSQL"];
 // NSLog(@"dir:%@",preferredDir);
  
  NSSavePanel *savePanel = [NSSavePanel savePanel];
  int choice = [savePanel runModalForDirectory:preferredDir file:@"EditWindow.sql"];
  if(choice == NSFileHandlingPanelOKButton)
   {
	if(result = [[inputTextView string] writeToFile:[savePanel filename] atomically:YES])
	  {
       tmpString =[NSString stringWithString:@"Write was successfull!"];
       }
     else {
      tmpString = [NSString stringWithString:@"Write Failed!"]; 
      }
    // Tell the user what happened  
    NSRunInformationalAlertPanel(@"Status Of Save Operation",tmpString,@"OK",nil,nil);

   }
  
}

- (IBAction)open:(id)sender
{
    NSOpenPanel *op = [NSOpenPanel openPanel];
    int returnCode = [op runModalForTypes:nil];
    if (returnCode != NSOKButton) {
        return;
    }
    NSString *path = [op filename];
    NSString *contents = [NSString stringWithContentsOfFile:path];
    [inputTextView setString:contents];
    [inputTextView display];
}


- (void)showTuples
{
    int fields = PQnfields(lastResult);
    int difference;
    int i;
    NSTableColumn *currentColumn;
    NSArray *oldColumns;
    
    // Get the right number of columns in the table view
    oldColumns = [tableView tableColumns];
    difference = fields - [oldColumns count];
    
    // Too many? remove some columns
    if (difference < 0) {
        for (i = [oldColumns count] - 1; i >= fields; i--) {
            [tableView removeTableColumn:[oldColumns objectAtIndex:i]];
        }
    }

    // Too few? add some columns
    if (difference > 0) {
        for (i=0; i < difference; i++) {
            currentColumn = [[NSTableColumn alloc] init];
            [currentColumn setEditable:YES];   //Changed from NO JHL
            [currentColumn setMinWidth:70.0];
			[[currentColumn  dataCell] setDrawsBackground:NO];
			[[currentColumn dataCell] setFont:[tableView currentFont]];
            [tableView addTableColumn:currentColumn];  // jhl
            [currentColumn release];
        }
    }
    
    // Relabel the columns
    oldColumns = [tableView tableColumns];
    for (i = 0; i < [oldColumns count]; i++) {
        NSString *fieldString;
        char *nameOfField;
        nameOfField = PQfname(lastResult, i);
        fieldString = [NSString stringWithUTF8String:nameOfField];
        
        // Get the column to label
        currentColumn = [oldColumns objectAtIndex:i];
        [currentColumn setIdentifier:fieldString];
        [[currentColumn headerCell] setStringValue:fieldString];
        // We will use this tag to figure out which column
        // this represents.  (Important if the columns are
        // reordered!)
        [[currentColumn headerCell] setTag:i];

    }
	//[tableView setUsesAlternatingRowBackgroundColors:YES];
}

- (void)executeSQL:(NSString *)sql
{
    PGresult *result;
    ExecStatusType resultStatus;
    NSString *logString;

    // Run the SQL
    result = PQexec(connection, [sql UTF8String]);
    
    // Check the status
    resultStatus = PQresultStatus(result);
    
    // OK, with rows
    if (resultStatus == PGRES_TUPLES_OK) {
        int ntuples = PQntuples(result);
        int fields = PQnfields(result);
        logString = [NSString stringWithFormat:@"%d records, %d fields\n", ntuples, fields];
        [self appendToLog:logString color:[NSColor blueColor]];
        [self setLastResult:result];
        return;
    }
        
    // Bad
    if (resultStatus == PGRES_NONFATAL_ERROR) {
        logString = [NSString stringWithFormat:@"Non-fatal - %s", PQresultErrorMessage(result)];
        [self appendToLog:logString color:[NSColor orangeColor]];
		PQclear(result);
		return;
    }
    
    // Real bad
    if (resultStatus == PGRES_FATAL_ERROR) {
         logString = [NSString stringWithFormat:@"Fatal - %s", PQresultErrorMessage(result)];
        [self appendToLog:logString color:[NSColor redColor]];
		PQclear(result);
		return;
    }
    
	// Return some more info to the log
	if(strlen( PQcmdStatus(result))) {
	   [self appendToLog:[NSString stringWithFormat: @"%s\n",PQcmdStatus(result)] color:[NSColor blueColor]];
	}
	
	if( strlen(PQcmdTuples(result))>0) {
	   [self appendToLog:[NSString stringWithFormat: @"%s rows affected.\n",PQcmdTuples(result)] color:[NSColor blueColor]];
	}
	
	//NSLog(@"Status: Status: %s Tuples: %s",PQcmdStatus(result),PQcmdTuples(result));
    PQclear(result);
}

- (IBAction)execute:(id)sender
{
    int i, count;
    [self clearLog];
    [self setLastResult:NULL];
    NSArray *listOfStatements;
	NSString * sql ;
	NSRange selRange;   // location =NSNotFound if no selection
	// vars for recording lapsed time
	struct timeval start,fin;
	double lapsed;
	long seconds, usecs;
	
	// We will give priority to any selection in the window
	selRange = [inputTextView selectedRange];
	if(selRange.length)
	{
	  sql = [[[inputTextView string] substringWithRange:selRange] copy];
	}
	else {sql = [[inputTextView string] copy]; }
    
    // Read the input and add it to the history
    [executedStatements addObject:sql];
    [sql release];
    currentIndex = [executedStatements count] - 1;
    
    // Start the progress indicator (in case of a long operation)
    [progressIndicator startAnimation:nil];
    
    // Break up the statements
    // FIXME: this won't handle escaped ; correctly.
    //listOfStatements = [sql componentsSeparatedByString:@";"];
	listOfStatements =(NSArray*) [self parseInput:sql];

    // Execute each of the statements
    count = [listOfStatements count];

    
    gettimeofday(&start,0);   // get starting time
    for (i=0; i< count; i++) {
        NSString *currentStatement;
        currentStatement = [listOfStatements objectAtIndex:i];
        
        // execute the line
        [self executeSQL:currentStatement];
    }
    gettimeofday(&fin,0);  // get finish time
	
    if (lastResult) {
        [self showTuples];
    }
    // do lapsed time calculation
	  seconds = fin.tv_sec-start.tv_sec;
      usecs =   fin.tv_usec-start.tv_usec;
      if(usecs < 0) {
         seconds-- ; usecs = usecs +1000000;
       }
	   
	lapsed = (double)seconds * 1000.0 + (double)usecs*0.001;
    
    [self appendToLog:[NSString stringWithFormat: @"Done in %d milliseconds.\n", (long)lapsed] color:[NSColor blackColor]];
    
    // update tableView
    [tableView reloadData];
    
    // Stop progress indicator
    [progressIndicator stopAnimation:nil];

}
- (void)disconnect
{
    // At termination,  disconnect gracefully
    // NSLog(@"Disconnected");
    PQfinish(connection);
    connection = NULL;
}

- (IBAction)connect:(id)sender
{
    // Read the text fields,  skip empty ones
    NSMutableString *connString = [NSMutableString string];
    NSString *input = [userTextField stringValue];
    if ([input length]) 
        [connString appendFormat:@"user=%@ ", input];

    input = [passwordTextField stringValue];
    if ([input length]) 
        [connString appendFormat:@"password=%@ ", input];
    
    input = [dbTextField stringValue];
    if ([input length]) 
        [connString appendFormat:@"dbname=%@ ", input];

    input = [hostTextField stringValue];
    if ([input length]) 
        [connString appendFormat:@"host=%@ ", input];

    //NSLog(@"connection string:\'%@\'", connString);
    
    connection = PQconnectdb([connString UTF8String]);
    
    // Bring up an alert panel to show status
    if (PQstatus(connection) == CONNECTION_OK) {
	
	   // record new defaults
	   [prefs setObject:[userTextField stringValue] forKey:@"BSQLuser"];
	   [prefs setObject:[hostTextField stringValue] forKey:@"BSQLhost"];
	   [prefs setObject:[dbTextField stringValue] forKey:@"BSQLdb"];

      //  NSRunAlertPanel(@"Good", @"Connection is up", nil, nil,nil);
        [loginSheet orderOut:nil];
        NSString * successStr = [NSString stringWithFormat:@"Connected to DB:%@ as user:%@ on host:%@ at %@\n",
		    [dbTextField stringValue], [userTextField stringValue], [hostTextField stringValue],
			[[NSDate date] description] ];
		[self appendToLog:successStr color:[NSColor magentaColor]];
		[self appendToLog:@"Ready...\n" color:[NSColor blackColor]];
        // Prepare for executing and logging for connection
        [self fetchTypes];
        PQsetNoticeProcessor(connection, handle_notice, self);
        
        // Hide the sheet
        [NSApp endSheet:loginSheet];
		NSArray *theCols =	[tableView tableColumns];
	    NSFont * currentFont = [[[theCols objectAtIndex:0] dataCell] font];
		[tableView setCurrentFont:currentFont];
		// create the outline view controller 
		if(outlineViewController) // we have already come through
		{
		  OLDataSource *tmp = outlineViewController; 
		  outlineViewController = [[OLDataSource alloc] initWithConnections: connection
		                           theView:outlineView menuState:[showHideItem state]];
		  [tmp release];
		 }
		 else {
		   outlineViewController = [[OLDataSource alloc] initWithConnections: connection 
		                                theView:outlineView menuState:[showHideItem state]];										
		  [self buildScriptMenu:scriptMenu];
		}
		
    }  else {
        NSRunAlertPanel(@"Bad", @"Connection not up = %d", nil, nil,nil, PQstatus(connection));
        
        // Give them another chance (clear connection, don't hide sheet)
        [self disconnect];
    }
}

- (void)awakeFromNib
{
    // Set attributes of the progress indicator
    [progressIndicator setStyle:NSProgressIndicatorSpinningStyle];
    [progressIndicator setDisplayedWhenStopped:NO];
    
	// fill the login sheet with the current defaults
	[userTextField setStringValue: [prefs stringForKey:@"BSQLuser"]];
	[hostTextField setStringValue: [prefs stringForKey:@"BSQLhost"]];
	[dbTextField setStringValue: [prefs stringForKey:@"BSQLdb"]];
	
	// set the input window font
	NSData * inputFontAsData = [prefs objectForKey:@"BSQLinputFont"];
	NSFont * inputFont = [NSKeyedUnarchiver unarchiveObjectWithData:inputFontAsData];
	[inputTextView setFont:inputFont];
	
	// set the table font
	NSData * tableFontAsData = [prefs objectForKey:@"BSQLtableFont"];
	NSFont * tableFont = [NSKeyedUnarchiver unarchiveObjectWithData:tableFontAsData];
	// we created the table view with at least one column
	[self firstTableViewSetup:tableFont];
	
    // Immediately show login panel
    [NSApp beginSheet:loginSheet	
        modalForWindow:mainWindow 
        modalDelegate:self 
        didEndSelector:NULL 
        contextInfo:NULL];
        
    // Handle tool tips for table view explicitly
    [self registerToolTipHandler];
	
	// JHL
	[outputTextView setFont:[NSFont fontWithName:@"Courier" size:12]];
	[tableView setDoubleAction:@selector(whereAmI)];
	[outlineView setDoubleAction:@selector(showSelTable)];
	//[self buildScriptMenu:scriptMenu];
	//[mainWindow setController:self];
}

-(IBAction) updateScriptsMenu:(id)sender
{
  int i;
  NSMenu * zMenu = [sender menu];
  int menuSize = [ zMenu numberOfItems];
  // destroy the existing menu
  for( i=3 ; i< menuSize ; i++) { 
	 [zMenu removeItemAtIndex:3];	  
	  }
  // now rebuild the menu
  [ self buildScriptMenu: zMenu];
}

-(void) buildScriptMenu: (NSMenu*)theMenu
 {
 	BOOL isDir,res;
   // NSString *cmd= @"tell app \"Finder\" to open posix file \"/Users/jerry/Documents/SQL Stuff\"";
   NSFileManager *manager = [NSFileManager defaultManager];
   NSString *dirPath = [@"~/Library/BiggerSQL" stringByExpandingTildeInPath];
   //NSLog(@"dirPath=%@",dirPath);
   if(! ([manager fileExistsAtPath:dirPath isDirectory:&isDir] && isDir))
    { 
	  res = [manager createDirectoryAtPath:dirPath attributes:nil];
	  
	}
    // Here we have found or created the directory
    [self extendScriptMenu: theMenu withPath:dirPath];

 }
 
-(void) extendScriptMenu: (NSMenu*) menu withPath: (NSString*)thePath 
{
  BOOL isDir;
  NSString *dirPath;
  NSFileManager *manager = [NSFileManager defaultManager];
  NSArray *dirArray = [manager directoryContentsAtPath:thePath];
  NSEnumerator *direnum = [dirArray objectEnumerator];
  NSString *pname;
  while (pname =(NSString*) [direnum nextObject]) {
   // Is it a directory?
   dirPath = [NSString stringWithFormat:@"%@/%@",thePath,pname];
   if( ([manager fileExistsAtPath:dirPath isDirectory:&isDir] && isDir))
    {
	  NSMenuItem *tmp = [menu addItemWithTitle:pname action:nil keyEquivalent:@""];
	  NSMenu *newMenu = [[NSMenu alloc] initWithTitle:pname];
	  [menu setSubmenu:newMenu forItem:tmp];
	  [newMenu release ];
	  [self extendScriptMenu: newMenu withPath:dirPath];
	}
	else
    if ([[pname pathExtension] isEqualToString:@"sql"])
	{
	   // if ( [pname characterAtIndex:0] == (unichar)'.') continue; // skip dot files
        NSMenuItem *tmp =  [menu addItemWithTitle:pname action:@selector(loadScript:) keyEquivalent:@""];
		[tmp setRepresentedObject:dirPath];  
	 }
  }
}

// tell the controller to show the selected table
-(void)showSelTable
{ 
  // NSLog(@"ShowTable");
  int theRow = [outlineView clickedRow];
  if (theRow >= 0){
    Node * selectedItem = [outlineView itemAtRow:theRow];
   // NSLog(@" %@ -- %@ -- %@", [selectedItem itemName], [selectedItem itemInfo],[selectedItem moreItemInfo]);
   if([[selectedItem moreItemInfo] isNotEqualTo:@"<data type>"])  //it is a table/view
    {
	   NSString *workString = [NSString stringWithFormat:@"select * from %@.%@",[selectedItem itemInfo],
	       [selectedItem itemName]];
	  [inputTextView setString: workString];
	  [self execute:self];
	} else 
	   { [inputTextView insertText: [[selectedItem itemName] stringByAppendingString: @", "]];
	     [mainWindow makeFirstResponder:inputTextView];
	   } 
 }
}

// Table view datasource methods

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
    if (lastResult)
        return PQntuples(lastResult);
    else 
        return 0;
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
    char *value;
    NSString *dataString;
    int fieldNum = [[tableColumn headerCell] tag];
    BOOL isNull = PQgetisnull(lastResult, row, fieldNum);
    if (isNull) {
	  if( [displayNULL state])
	      { return @"<NULL>" ; }
	  else  return @"" ;
    }
    value = PQgetvalue(lastResult, row, fieldNum);
    dataString = [NSString stringWithUTF8String:value];
	
	//[[tableColumn  dataCell] setDrawsBackground:NO];
	
    return dataString;
}

// Diconnect gracefully when the user quits

- (void)applicationWillTerminate:(NSNotification *)notification
{
    [self disconnect];
}

// allow the user to open by dragging a file onto the icon...

- (BOOL) application: (NSApplication*) anApplication
	    openFile: (NSString*     ) aFileName
{
  [inputTextView setString:[NSString stringWithContentsOfFile:aFileName]];  
  return YES;
}

// Allow the user to create another connection
- (IBAction)reconnect:(id)sender
{
    [self disconnect];
    
    // This will bring up the login sheet again
    [self awakeFromNib];
}

// History methods

- (IBAction)previous:(id)sender
{
    if (currentIndex == 0) {
        NSBeep();
        return;
    }
    currentIndex--;
    NSString *statement = [executedStatements objectAtIndex:currentIndex];
    [inputTextView setString:statement];
    [inputTextView display];
}
- (IBAction)next:(id)sender
{
    if (currentIndex >= [executedStatements count] - 1) {
        NSBeep();
        return;
    }
    currentIndex++;
    NSString *statement = [executedStatements objectAtIndex:currentIndex];
    [inputTextView setString:statement];
    [inputTextView display];
}

// rebuild the outline view information
- (IBAction)refresh:(id)sender
{
    OLDataSource * tmp = outlineViewController;	
	outlineViewController = [[OLDataSource alloc] initWithConnections: connection
	                            theView:outlineView menuState: [showHideItem state]];
	[tmp release];

}
 
- (IBAction)saveSelection:(id)sender {
  [myWriter saveAction:lastResult];
}   
- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
{
    // Disable next and previous menu items as 
    // necessary
    if ([menuItem action] == @selector(next:)) {
        return (currentIndex < [executedStatements count] - 1);
    }
    if ([menuItem action] == @selector(previous:)) {
        return (currentIndex > 0);
    }
    return YES;
}

- (IBAction)openScriptsDir:(id)sender
{
   BOOL isDir,res;
   NSFileManager *manager = [NSFileManager defaultManager];
   NSString *dirPath = [@"~/Library/BiggerSQL" stringByExpandingTildeInPath];
  // NSLog(@"dirPath=%@",dirPath);
   if(! ([manager fileExistsAtPath:dirPath isDirectory:&isDir] && isDir))
    { 
	  res = [manager createDirectoryAtPath:dirPath attributes:nil];
	  
	}
  // NSLog(@"res:%i",res);
   NSString *cmd = [ [@"tell app \"Finder\" \n activate \nopen posix file \"" stringByAppendingString:dirPath]
                       stringByAppendingString:@"\"\n end"];
  // NSLog(@"cmd:%@",cmd);
   NSDictionary *err= [[NSDictionary alloc] init];	
   NSAppleScript *myScript = [[NSAppleScript alloc] initWithSource:cmd];
   NSAppleEventDescriptor * result =[myScript executeAndReturnError: &err];
   if(result){}; // surpress warning....
   [myScript release];
   [err release];

}

-(IBAction)loadScript:(id)sender
{
//     NSString *dirPath = [@"~/Library/BiggerSQL" stringByExpandingTildeInPath];
//	 NSString *theFile = [[dirPath stringByAppendingString:@"/"] stringByAppendingString: [sender title]];
//    [inputTextView setString:[NSString stringWithContentsOfFile:theFile]];
//    [inputTextView display];	
    NSString * fileName = [NSString stringWithFormat:@"-- %@\n\n%@",
	    [sender representedObject],[NSString stringWithContentsOfFile:[sender representedObject]]];
	[inputTextView setString:fileName];
//	[inputTextView setString:[NSString stringWithContentsOfFile:[sender representedObject]]];
    [inputTextView display];
 
}
// For types in tool tips,  used fetched type names + OIDs

- (NSString *)typeNameForOID:(Oid)typeOID
{
    int ntuples = PQntuples(typeResult);
    int oidForRow;
    char *oidAsString;
    int i;
    for (i=0; i < ntuples; i++) {
        oidAsString = PQgetvalue(typeResult, i, 0);
        oidForRow = atoi(oidAsString);
        if (oidForRow == typeOID) {
            return [NSString stringWithFormat:@"%s",
                            PQgetvalue(typeResult, i, 1)];
        }
    }
    return @"Not found";
}
- (IBAction)quit:(id)sender
{
    [NSApp endSheet:loginSheet];
    [NSApp terminate:nil];
}


// Methods to deal with tooltips for table view

- (NSString *)view:(NSView *)view stringForToolTip:(NSToolTipTag)tag point:(NSPoint)point userData:(void *)data
{
    NSTableColumn *tc;
    int columnIndex = [tableView columnAtPoint:point];
    if (columnIndex >= 0) {
        tc = [[tableView tableColumns] objectAtIndex:columnIndex];
        int tag = [[tc headerCell] tag];
        Oid typeOID = PQftype(lastResult,tag);
        if (typeOID == 0) {
            return nil;
        }
        return [self typeNameForOID:typeOID];
    }
    return nil;
}

- (void)registerToolTipHandler
{
    int count, i;
    
    [tableView removeAllToolTips];
    
    count = [[tableView tableColumns] count];
    for (i=0; i<count; i++) {
        [tableView addToolTipRect:[tableView rectOfColumn:i] owner:self userData:nil];
    }
}

- (void)windowDidResize:(NSNotification *)notification
{
    [self registerToolTipHandler];
}

- (void)splitViewDidResizeSubviews:(NSNotification *)notification
{
    [self registerToolTipHandler];
}

// we want to handle the "enter" key in a special way so we have to make ourself the
// delegate of inputTextView
- (BOOL)textView:(NSTextView *)textView doCommandBySelector:(SEL)commandSelector
{
  NSRange selRange,aRange;
   if( @selector(insertNewline:) == commandSelector)
   {
     // shift/return or enter?
	 /*
     if( !( ( [[NSApp currentEvent] keyCode] == ENTER_KEY_CODE) ||
			 ([[NSApp currentEvent] keyCode] == RETURN_KEY_CODE) && 
			 ([[NSApp currentEvent] modifierFlags] & NSShiftKeyMask )
			)
		)  return NO; 
	 */
	 NSString *characters = [[NSApp currentEvent] characters];
	 unichar ch = [characters characterAtIndex:0];
	 
	 if( !( (ch == NSEnterCharacter) || 
	        (ch == NSCarriageReturnCharacter) &&
			([[NSApp currentEvent] modifierFlags] & NSShiftKeyMask)
		   )
		)   return NO ;
		
	 // OK, we have the enter or shift return key pressed, first check to see if there is a selection
	 // and if so, execute the  selection.
	 selRange = [inputTextView selectedRange];  // get the selection bounds.
	 if(selRange.length != 0)  // We have a selection...go execute it.
	  { 
	     [self execute: self];  // we invoke the menu handler
		 return YES;            // we handled the key
	  }
	  // if we get here we don't have a selection so we will try to execute the
	  // line containing the cursor ( assuming the line containing the cursor is not empty).
	  // First get the line containing the cursor.
	  aRange =[[inputTextView string] lineRangeForRange: selRange];
	  if (aRange.length==0) // we are on an empty line, do nothing
	   {
	      return NO;
		}
	  // here aRange.length > 0, so we have a real line, so we will select it and call the execute function
	  [inputTextView setSelectedRange:aRange];
	  [self execute: self];
	  [inputTextView setSelectedRange:selRange];  // restore the cursor
	  return YES;   // we handled the key
	}
	return NO;
 }
 
 -(NSArray *) parseInput: (NSString *) statementString 
{
   NSString *sep = @"\001";
   int length,i;
   BOOL inQuote;             // true when in a quoted string
   NSMutableString *workingString;  
   NSRange replace;
   
   workingString = [NSMutableString stringWithString:statementString];
   length = [workingString length];
   inQuote = NO;
  if([sendSingleStatements state] == NSOnState)   
   for(i=0; i< length; i++)
   {   
       if([workingString characterAtIndex:i] == '\'') {
	        inQuote = (inQuote)?NO:YES;
			continue;
	    }
		if( [workingString characterAtIndex:i] == ';' && !inQuote) {
		    replace = NSMakeRange(i,1);
			[workingString replaceCharactersInRange:replace withString: sep];
		 }
	 }
	 //NSLog(@"Parse Results: %@", [workingString componentsSeparatedByString:sep]);

	 return [workingString componentsSeparatedByString:sep] ;
}
-(void) openDraggedFile: (NSString*) theFile 
{
	NSString *contents = [NSString stringWithContentsOfFile:theFile];
    [inputTextView setString:contents];
    [inputTextView display];
}

// toggle displaying null as <NULL>
-(void)displayNullField:(id)sender
{
      if([sender state] == NSOnState)
     {
	   [sender setState:NSOffState];
	 }
	 else
	 {
	   [sender setState:NSOnState];
	 }
}

// determine whether or not to display the system tables
-(void)showHideSystemTables:(id)sender
{  // set the new state
   if([sender state] == NSOnState)
     {
	   [sender setState:NSOffState];
	 }
	 else
	 {
	   [sender setState:NSOnState];
	 }
	// rebuild the outline view 
	OLDataSource * tmp = outlineViewController;	
	outlineViewController = [[OLDataSource alloc] initWithConnections: connection
	                            theView:outlineView menuState: [sender state]];
    [tmp release];
}

-(void)sendSingleStatements:(id)sender
{
   // set the new state
   if([sender state] == NSOnState)
     {
	   [sender setState:NSOffState];
	 }
	 else
	 {
	   [sender setState:NSOnState];
	 }
} 
-(IBAction)setEditWindowFont:(id)sender
{
 
  NSFontPanel *myFontPanel = [NSFontPanel sharedFontPanel];
  [myFontPanel setDelegate:self];
  [myFontPanel setPanelFont:[inputTextView font] isMultiple:NO];
  [myFontPanel setEnabled:YES];
  [myFontPanel makeKeyAndOrderFront:self];  
  
}
-(BOOL)windowShouldClose:(id)sender
{
  [mainWindow makeFirstResponder:inputTextView];
  return YES;
}

-(IBAction)setTableViewFont:(id)sender
{
  [mainWindow makeFirstResponder:tableView];  
  NSFontPanel *myFontPanel = [NSFontPanel sharedFontPanel];
  [myFontPanel setDelegate:self];
  [myFontPanel setPanelFont:[tableView currentFont] isMultiple:NO];
  [myFontPanel makeKeyAndOrderFront:self];  
}

-(void) firstTableViewSetup:(NSFont*)aFont
{
   int i;
   NSArray *theCols = [tableView tableColumns];
   int colCnt = [tableView numberOfColumns];
   for(i=0; i< colCnt; i++)
  {   
   [[[theCols objectAtIndex:i] dataCell] setFont:aFont];
  }
   [tableView setRowHeight:[[[theCols objectAtIndex:0] dataCell] cellSize].height]; //[self tile];
   [tableView setCurrentFont:aFont];
}
@end
