#import "OLDataSource.h"

@implementation OLDataSource

// init is like a constructor
- initWithConnections:(PGconn*) theConnection theView:(id) tableOutline menuState:(int)showTable
{
    [super init];
    rootNode = [[Node alloc] init];
    [rootNode setItemName: @"Tables"];
	[rootNode setItemInfo: @""];
	[rootNode setMoreItemInfo:@""];
	menuState = showTable;        // record the state of the menu...
	connection = theConnection;   // remember the database connection
	outlineView = tableOutline;   // remember the outlineView connection
	[self buildTableList:rootNode];
	[outlineView setDataSource:self]; // we do the work...
    return self;
}

// an action method
- (IBAction)addChild:(id)sender
{
    // Which row is selected?
    int selectedRow = [outlineView selectedRow];
    Node *selectedItem;
    Node *newNode;
    
    // Is nothing selected?
    if (selectedRow < 0) {
        NSBeep();
        return;
    }
    
    // Which node is that?
    selectedItem  = [outlineView itemAtRow:selectedRow];
    
    // Create a new child node
    newNode =  [[Node alloc] init];
    [newNode setItemName:@"New child"];
    [selectedItem addChild: newNode];    
    // Parent node is retaining the new node
    [newNode release];

    // Brute force reload to update sums
    [outlineView reloadItem:rootNode reloadChildren:YES];
}

// Data source methods get called automatically

// This method is called repeatedly when the table view is displaying it self. 
- (id)outlineView:(NSOutlineView *)ov child:(int)index ofItem:(id)item
{
    // is the parent non-nil?
    if (item)
        // Return the child
        return [item childAtIndex:index];
    else 
        // Else return the root
        return rootNode;
}

// Called repeatedly to find out if there should be an
// "expand triangle" next to the label
- (BOOL)outlineView:(NSOutlineView *)ov isItemExpandable:(id)item
{
    // Returns YES if the node has children
    return [item expandable];
}

// Called repeatedly when the table view is displaying itself
- (int)outlineView:(NSOutlineView *)ov numberOfChildrenOfItem:(id)item
{
    if (item == nil) {
        // The root object;
        return 1;
    }
	//NSLog(@"->%@",[item itemName]);
    return [item childrenCount];
}

// This method gets called repeatedly when the outline view is trying
// to display it self.

- (id)outlineView:(NSOutlineView *)ov 
    objectValueForTableColumn:(NSTableColumn *)tableColumn 
    byItem:(id)item
{
    // I set the identifier of the columns in IB's inspector
    NSString *identifier = [tableColumn identifier];
    // What is returned depends upon which column it is
    // going to appear.
    if ([identifier isEqual:@"itemName"])
        return [item itemName];
     
	 if([identifier isEqual:@"itemInfo"])  	
	     return [item itemInfo];
		 
	// if([identifier isEqual:@"moreItemInfo"])
	     return [item moreItemInfo];
    
}

// This method gets called when the user edits the field.
/*
- (void)outlineView:(NSOutlineView *)ov 	
    setObjectValue:(id)object 
    forTableColumn:(NSTableColumn *)tableColumn 
    byItem:(id)item
{
    // The only editable column in the item name column
    // so I know immediately where to put it.
    // If there were more editable columns I would
    // need an if-statement like in the previous method
    [item setItemName:object];
    
    // Brute force reload to update sums
    [outlineView reloadItem:rootNode reloadChildren:YES];
}
*/

-(void)buildTableList: (Node*) schemaNode
{
   //NSString *username;
  // NSString *tablename;
   PGresult *result;
   NSString *dataString;
   NSMutableString *sql;
   Node *newNode;
   int i;
   char *value;
   
  // sql = [NSMutableString stringWithFormat:@"%s'%s'","select schemaname, tablename from pg_tables where tableowner=",
  //            PQuser(connection)];
  if(menuState)
  { // long listing?
  sql = [NSMutableString stringWithFormat:@"%s",
          "select table_schema, table_name,table_type from information_schema.tables order by table_schema desc, table_type asc"];
   }
   else
   {
   sql = [NSMutableString stringWithFormat:@"%s",
     "select table_schema, table_name,table_type from information_schema.tables \
	  where table_schema != 'pg_catalog' and table_schema != 'information_schema' \
	  order by table_schema desc, table_type asc"];
   }
   result = PQexec(connection, [sql UTF8String]);
        if (PQresultStatus(result) != PGRES_TUPLES_OK)
        {
                NSLog( @" failed: %s", PQerrorMessage(connection));
                PQclear(result);
        }

        /* Get Table Count, better be 3 */
        //int nFields = PQnfields(result);

        /* next, add the table names and info for each table */
        for (i = 0; i < PQntuples(result); i++)
        {
				   newNode = [[Node alloc] init];
				   
				   value = PQgetvalue(result,i,0);    // schema name
				   dataString = [NSString stringWithUTF8String:value];
				   [newNode setItemInfo:dataString];
				   
				   value = PQgetvalue(result,i,1);    // table name
				   [newNode setItemName:[NSString stringWithUTF8String:value]];
				   
				   value = PQgetvalue(result,i,2);    // table type, base or view
				   [newNode setMoreItemInfo:[NSString stringWithUTF8String:value]];
		   
				   // get info about the current table
				   [self buildTableInfo: newNode];
				   // add the table name and info to the parent
                   [schemaNode addChild: newNode];
				   
				   [newNode release];
				
        }

        PQclear(result);
 }
 
 -(void)buildTableInfo: (Node*) tableNode
 {
   PGresult * result;
   Node *newNode;
   int i ;
   NSMutableString *sql;
   
   sql = [ NSMutableString stringWithFormat:@"%s'%@' %s",
        "SELECT a.attname, format_type(a.atttypid, a.atttypmod) FROM pg_class c, pg_attribute a WHERE c.relname = ",
		[tableNode itemName],
        "AND a.attnum > 0 AND a.attrelid = c.oid ORDER BY a.attnum"];
		
   // get the info...
   result = PQexec(connection, [sql UTF8String]);
        if (PQresultStatus(result) != PGRES_TUPLES_OK)
        {
                NSLog( @" failed: %s", PQerrorMessage(connection));
                PQclear(result);
        }

	/* Get field Count, should be 2 */
	//int nFields = PQnfields(result);
        /* next, add the column names and datatype for each table */
		
        for (i = 0; i < PQntuples(result); i++)
        {
				  // dataString = [NSString stringWithFormat: @"%s : %s",PQgetvalue(result,i,0),PQgetvalue(result,i,1)];
				   newNode = [[Node alloc] init];
				   [newNode setItemName:[NSString stringWithUTF8String:PQgetvalue(result,i,0) ]];
				   [newNode setItemInfo:[NSString stringWithUTF8String:PQgetvalue(result,i,1) ]];
				   [newNode setMoreItemInfo:@"<data type>"];				   
                   [tableNode addChild: newNode];
				   [newNode release];
        }

        PQclear(result);
 }
 
- (void)dealloc
{
    [rootNode release];
    [super dealloc];
}
@end
